﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Configuration;
using Microsoft.Win32;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;

namespace raktajino
{
   
    #region Products

    /// <summary>
    /// Products - Container for all available Packages
    /// </summary>
    public class Products
    {
        public List<Package> Packages = new List<Package>();
        XmlDocument XmlInstallRequirements = new XmlDocument();

        public Products(XmlDocument HostingOptions)
        {
            // fill the InstallRequirements doc
            string urlInstallationRequirements = ConfigurationSettings.AppSettings["InstallationRequirementsURL"];
            if (urlInstallationRequirements == string.Empty)
            {
                string ExMsg = string.Format("No installationRequirements url found!");
            }

            try
            {
                XmlInstallRequirements.Load(urlInstallationRequirements);
            }
            catch (SystemException ex)
            {
                string ExMsg = string.Format("A problem occurred when loading the installationRequirements: {0}", ex.Message);
            }
        
            XmlNodeList xmllistproducts = HostingOptions.DocumentElement.SelectNodes("//Product");

            for (int i = 0; i < xmllistproducts.Count; i++)
            {
                XmlNode xmlProduct = xmllistproducts[i].FirstChild;
                string PackageName = xmlProduct.InnerText;
                Package package = new Package(PackageName, HostingOptions, XmlInstallRequirements);
                this.Packages.Add(package);
            }
        }
    }

    /// <summary>
    /// Contains all actions required to install - including Installationcommands (
    /// </summary>
    public class InstallAction
    {
        public XmlNode XmlAction;
        public InstallationCommand Command;
        public bool done;
        public int ExitCode;
        public string ExitInfoString;

        public InstallAction()
        {
            this.ExitCode = -1000;
            this.done = false;
            this.XmlAction = null;
            this.Command = null;
            this.ExitInfoString = string.Empty;
        }
    }

    /// <summary>
    /// Package - Contains all information necessary to run installation process
    /// </summary>
    public class Package
    {
        public string Name;
        public string Displaytext;
        public string EulaUrl;
        public string DescriptionUrl;
        public string SelfCheck;
        public List<string> CategoryList = new List<string>();
        public XmlNode ProductXmlNode;
        public string InstallationUrl;
        public UserInput UserInput;
        public Uri DownloadUrl;

        public InstallationRequirements InstallationRequirements;
        private XmlDocument XmlInstallRequirements = null;
        public List<InstallAction> InstallationActions = new List<InstallAction>();

        public Package(string PackageName, XmlDocument HostingOptions, XmlDocument XmlInstallRequirements)
        {
            this.Name = PackageName;
            this.XmlInstallRequirements = XmlInstallRequirements;
            string Productsearchpath = string.Format("//Product[Title='{0}']", this.Name);

            XmlNode XmlProduct = HostingOptions.SelectSingleNode(string.Format("{0}", Productsearchpath));
            this.ProductXmlNode = XmlProduct;

            Uri uri = null;
            //check the InstallURL if it is a http address if so then set CommandURI to currentdirctory\downloads.
            try
            {
                uri = new Uri(this.ProductXmlNode.SelectSingleNode("InstallURL").InnerText);
                if (uri.IsFile)
                {
                    this.InstallationUrl = uri.ToString();
                }
                else
                {
                    if (!uri.IsAbsoluteUri)
                    {
                        //throw new InvalidInputException(string.Format("InstallURL '{0}' is not an absolute uri \n I need e.g. 'http://myserver/test.msi'", uri.ToString()));
                        this.InstallationUrl = string.Empty;
                    }
                    else
                    {
                        //uri is valid hyperlink - comanduri is set to the downloaddirectory 
                        this.DownloadUrl = uri;
                        this.InstallationUrl = Environment.CurrentDirectory +  "\\" + ConfigurationSettings.AppSettings["DownloadSubDirectory"];
                    }
                }
            }
            catch (System.UriFormatException uriFex)
            {
                //throw new InvalidInputException(string.Format("InstallURL  at '{0}' is not a valid uri", this.Name));
                this.InstallationUrl = string.Empty;
            }


            try
            {
                this.Displaytext = string.Format("{0}<hr/>", XmlProduct.SelectSingleNode(string.Format("{0}/Description", Productsearchpath)).InnerText);
            }
            catch
            {
                this.Displaytext = string.Empty;
            }

            try
            {
                this.DescriptionUrl = string.Format("{0}", XmlProduct.SelectSingleNode(string.Format("{0}/DescriptionUrl", Productsearchpath)).InnerText);
            }
            catch
            {
                this.DescriptionUrl = string.Empty;
            }

            XmlNodeList xmllistcategories = XmlProduct.SelectNodes(string.Format("{0}/Categories/Category", Productsearchpath));
            for (int ii = 0; ii < xmllistcategories.Count; ii++)
            {
                string categoryEntry = xmllistcategories[ii].InnerText;
                CategoryList.Add(categoryEntry);
            }

            try
            {
                this.EulaUrl = string.Format("{0}", XmlProduct.SelectSingleNode(string.Format("{0}/EulaUrl", Productsearchpath)).InnerText);
            }
            catch
            {
                this.EulaUrl = string.Empty;
            }

            try
            {
                this.SelfCheck = string.Format("{0}", XmlProduct.SelectSingleNode(string.Format("{0}/Requirements/SelfCheck/Requirement", Productsearchpath)).InnerText);
            }
            catch
            {
                this.SelfCheck = string.Empty;
            }


            this.UserInput = new UserInput(XmlProduct.SelectNodes("UserInputs"));

            GetInstallationActions();

        }

        public void GetInstallationActions()
        {
            XmlNodeList xmlActions = this.ProductXmlNode.SelectNodes("Action");

            foreach (XmlNode singleAction in xmlActions)
            {
                InstallAction InstallationAction = new InstallAction();
                InstallationAction.XmlAction = singleAction;
                this.InstallationActions.Add(InstallationAction);
            }
        }

        public bool CheckRequirements()
        {
            this.InstallationRequirements = new InstallationRequirements(this.ProductXmlNode, this.XmlInstallRequirements);
            return this.InstallationRequirements.isValidInstallationRequirements;
        }

        public void GetInstallationCommand()
        {
            foreach (InstallAction singleInstallAction in this.InstallationActions)
            {
                InstallationCommand command = new InstallationCommand(this);
                command.CreateCommand(singleInstallAction.XmlAction, this.UserInput);
                singleInstallAction.Command = command;
            }

        }
    }

    /// <summary>
    /// InstallatioinCommand - builds the command that is executed to install the Package
    /// </summary>
    public class InstallationCommand
    {
        public string Command;
        public string Arguments;
        public Package Package;

        public InstallationCommand(Package Package)
        {
            this.Command = string.Empty;
            this.Arguments = string.Empty;
            this.Package = Package;
        }

        public void CreateCommand(XmlNode XmlAction, UserInput UserInput)
        {
            //string CommandURI = this.Package.InstallationUrl;
            string CommandType = XmlAction.SelectSingleNode("Type").InnerText;
            string filename = string.Empty;

            XmlNodeList XmlParameters = XmlAction.SelectNodes("Parameters/*");

            switch (CommandType.ToUpper())
            {
                case "EXE":
                    Command = Path.GetFullPath(ParseStringForEnvironmentVars(XmlAction.SelectSingleNode("Command").InnerText));
                    //filename = this.Command;

                    if (XmlParameters.Count > 0)
                    {
                        this.Arguments += string.Format("{0}", MakeParametersToArguments(UserInput, XmlParameters, " "));
                    }
                    this.Arguments = ParseStringForEnvironmentVars(this.Arguments);
                    break;

                case "CMD":
                    Command = string.Format(@"{0}\cmd.exe", Environment.GetFolderPath(Environment.SpecialFolder.System));
                    this.Arguments = string.Format(" /C {0}", XmlAction.SelectSingleNode("Command").InnerText);

                    if (XmlParameters.Count > 0)
                    {
                        this.Arguments += string.Format("{0}", MakeParametersToArguments(UserInput, XmlParameters, " "));
                    }
                    this.Arguments = ParseStringForEnvironmentVars(this.Arguments);
                    break;

                case "MSI":
                    filename = Path.GetFullPath(ParseStringForEnvironmentVars(XmlAction.SelectSingleNode("Command").InnerText));
                    Command = string.Format(@"{0}\msiexec.exe", Environment.GetFolderPath(Environment.SpecialFolder.System));
                    this.Arguments += string.Format(" /Package \"{0}\" ", filename);

                    if (XmlParameters.Count > 0)
                    {
                        this.Arguments += string.Format("{0}", MakeParametersToArguments(UserInput, XmlParameters, " "));
                    }
                    this.Arguments = ParseStringForEnvironmentVars(this.Arguments);
                    break;

                case "UNZIP":
                    //unzip is encapsulated in the code (you will find this logic in the winforms logic) - it is not an external proc that is to be launched
                    Command = "UNZIP";
                    if (XmlParameters.Count > 0)
                    {
                        this.Arguments += string.Format("{0}", MakeParametersToArguments(UserInput, XmlParameters, "@"));
                    }
                    this.Arguments = ParseStringForEnvironmentVars(this.Arguments);
                    break;


                default:
                    throw new InvalidInputException(string.Format("Command Type '{0}' not supported", CommandType));
                //break;
            }

        }

        #region Parameters
        private string MakeParametersToArguments(UserInput UserInput, XmlNodeList XmlParameters, string ArgumentDelimeterString)
        {
            string fullArgumentValue = string.Empty;
            if (string.IsNullOrEmpty(ArgumentDelimeterString))
            {
                ArgumentDelimeterString = " ";
            }

            foreach (XmlNode paramNode in XmlParameters)
            {
                //this is the hardcoded parameter with no userinput - we can see that as there are no attributes in the xml file
                if (paramNode.Attributes.Count == 0)
                {
                    fullArgumentValue += string.Format("{0}{1}",ArgumentDelimeterString,paramNode.InnerText);
                    continue;
                }
                //The others must be UserInput parameters
                foreach (UserInputParameter userInputParm in UserInput.Parameters)
                {
                    if (paramNode.Attributes["src"].Value == userInputParm.Name)
                    {
                        switch (userInputParm.InputType)
                        {
                            case UserInputType.uiTypeBool:
                                bool result = (bool)userInputParm.Result;
                                switch (result)
                                {
                                    case true:
                                        fullArgumentValue += string.Format("{0}{1}",ArgumentDelimeterString,paramNode.Attributes["valuetrue"].Value);
                                        break;
                                    case false:
                                        fullArgumentValue += string.Format("{0}{1}",ArgumentDelimeterString,paramNode.Attributes["valuefalse"].Value);
                                        break;
                                    default:
                                        break;
                                }
                                break;
                            case UserInputType.uiTypeString:
                                string XmlParameterValue = paramNode.Attributes["value"].Value;
                                fullArgumentValue += string.Format("{0}{1}",ArgumentDelimeterString,XmlParameterValue.Replace("%value", (string)userInputParm.Result));
                                break;
                            default:
                                break;
                        }
                    }
                }
            }
            return fullArgumentValue;
        }
        private string ParseStringForEnvironmentVars(string input)
        { 
           if (!input.Contains("%")) return input;

            Regex regex = new Regex("%[\\w]*%");
          
            MatchCollection mcol = regex.Matches(input);
            if (mcol.Count > 0)
            {
                foreach (Match m in mcol)
                {
                    switch (m.Value.ToUpper())
                    {
                        case "%INSTALLPATH%":
                            string filePath = this.Package.InstallationUrl;
                            input = input.Replace(m.Value, filePath);
                            break;
                        case "%TEMPSUBDIRECTORY%":
                            string tempdir = Environment.CurrentDirectory + "\\" + ConfigurationSettings.AppSettings["TempSubDirectory"];
                            if (!Directory.Exists(tempdir)) Directory.CreateDirectory(tempdir);
                            input = input.Replace(m.Value, tempdir);
                            break;
                        case "%TMP%":    //default(vista) C:\Users\bfrank\AppData\Local\Temp
                            input = input.Replace(m.Value, Environment.GetEnvironmentVariable("TMP"));
                            break;
                        case "%TEMP%":    //default(vista) C:\Users\bfrank\AppData\Local\Temp
                            input = input.Replace(m.Value, Environment.GetEnvironmentVariable("TMP"));
                            break;
                        case "%INSTALLLOGS%":    //default path to write the installation logs to
                            string folder = Environment.CurrentDirectory + "\\InstallLogs";
                            if (!Directory.Exists(folder)) Directory.CreateDirectory(folder);
                            input = input.Replace(m.Value, folder);
                            break;
                        case "%SYSTEMDRIVE%":     //default c:
                            input = input.Replace(m.Value, Environment.GetEnvironmentVariable("SYSTEMDRIVE"));
                            break;
                    }
                }
            }
            return input;
        }



        #endregion

        //private void AddHardcodedParameters(XmlNodeList XmlParameters)
        //{
        //    if (XmlParameters != null)
        //    {
        //        for (int i = 0; i < XmlParameters.Count; i++)
        //        {
        //            if (XmlParameters[i].Attributes.Count == 0)
        //            {
        //                this.Arguments += string.Format(" {0}", XmlParameters[i].InnerText);
        //            }
        //        }
        //    }
        //}
    }

    /// <summary>
    /// Categories contains a dictionary object in the format <Category> --- <Product><Title>,<Product><Title>,....
    /// </summary>
    public class Categories
    {
        public Dictionary<string, List<string>> categoryDictionary = new Dictionary<string, List<string>>();

        public Categories(XmlDocument HostingOptions)
        {
            XmlNodeList xmllistproducts = HostingOptions.DocumentElement.SelectNodes("//Product");
            List<string> allProductsList = new List<string>();

            for (int i = 0; i < xmllistproducts.Count; i++)
            {
                XmlNode Product = xmllistproducts[i].FirstChild;
                String PackageName = Product.InnerText;
                allProductsList.Add(PackageName);
                XmlNodeList xmllistcategories = HostingOptions.DocumentElement.SelectNodes(string.Format("//Product[Title='{0}']/Categories/Category", PackageName));
                List<string> packageCategories = new List<string>();
                for (int ii = 0; ii < xmllistcategories.Count; ii++)
                {
                    string categoryEntry = xmllistcategories[ii].InnerText;
                    packageCategories.Add(categoryEntry);
                }

                for (int ii = 0; ii < packageCategories.Count; ii++)
                {
                    List<string> packageNames = new List<string>();
                    if (!categoryDictionary.ContainsKey(packageCategories[ii]))
                    {
                        packageNames.Add(PackageName);
                        categoryDictionary.Add(packageCategories[ii], packageNames);
                    }
                    else
                    {
                        categoryDictionary.TryGetValue(packageCategories[ii], out packageNames);
                        packageNames.Add(PackageName);
                        categoryDictionary.Remove(packageCategories[ii]);
                        categoryDictionary.Add(packageCategories[ii], packageNames);
                    }
                }
            }
            categoryDictionary.Add("all", allProductsList);
        }
    }

    #endregion

    #region Installation

    public class InstallationRequirements
    {
        public string URL;
        /// <summary>
        /// True if Install Requirements specified in HostingOptions.xml
        /// </summary>
        public bool InstallCheckRequired = false;
        /// <summary>
        /// True if the Package could be installed
        /// </summary>
        public bool InstallCheckSuccessfull = false;
        public bool ProductAlreadyInstalled = false;

        //is true when the InstallationRequirements could be successfully verified 
        //returns false when a problem (e.g. checkfile not found) occurred
        private bool problemOccurred = false;
        public bool isValidInstallationRequirements
        {
            get { return !problemOccurred; }
        }

        public List<string> CheckResults = new List<string>();
        public List<string> MissingRequirements = new List<string>();
        public DateTime LastCheckTime;

        string urlInstallationRequirements = string.Empty;
        XmlDocument docInstallationRequirements = new XmlDocument();
        List<XmlNode> InstallationNodes = new List<XmlNode>();

        public InstallationRequirements(XmlNode Product, XmlDocument XmlInstallationRequirements)
        {
            XmlDocument docProduct = new XmlDocument();
            this.docInstallationRequirements = XmlInstallationRequirements;

            //set timestamp
            this.LastCheckTime = DateTime.Now;

            docProduct.LoadXml(Product.OuterXml);

            string ProductName = Product.FirstChild.InnerText;
            string searchpath = string.Format("//Product[Title='{0}']/Requirements", ProductName);
            string searchpathInstallRequirements = string.Format("{0}/InstallRequirements/*", searchpath);
            string searchpathSelfCheck = string.Format("{0}/SelfCheck", searchpath);

            #region Check Installation
            XmlNodeList Requirements = docProduct.SelectNodes(searchpath);

            if (Requirements.Count > 0)
            {
                InstallCheckRequired = true;
                //urlInstallationRequirements = ConfigurationSettings.AppSettings["InstallationRequirementsURL"];
                //if (urlInstallationRequirements == string.Empty)
                //{
                //    string ExMsg = string.Format("No installationRequirements url found!");
                //    this.CheckResults.Add(ExMsg);
                //    problemOccurred = true;
                //}

                //try
                //{
                //    docInstallationRequirements.Load(urlInstallationRequirements);
                //}
                //catch (SystemException ex)
                //{
                //    string ExMsg = string.Format("A problem occurred when loading the installationRequirements: {0}", ex.Message);
                //    this.CheckResults.Add(ExMsg);
                //    problemOccurred = true;
                //}

                XmlNodeList SelfCheckRequirements = docProduct.SelectNodes(searchpathSelfCheck);

                if (SelfCheckRequirements.Count > 0)
                {
                    if (InstallationReferenceCheck_Success(SelfCheckRequirements))
                    {
                        try
                        {
                            this.ProductAlreadyInstalled = CheckInstallationRequirements();
                        }
                        catch
                        {
                            this.problemOccurred = true;
                        }

                        this.CheckResults.Clear();
                        if (this.ProductAlreadyInstalled == true)
                        {
                            this.CheckResults.Add("Product is already found and only one instance of this product can be installed.\nIn order to reinstall you would have to remove the product first.");
                        }
                        else
                        {
                            this.CheckResults.Add("Product is not yet found on the system.");
                        }
                    }
                    else
                    {
                        string ExMsg = string.Format("Invalid Installation Requirement found in {0} (SelfCheckRequirements).\r\nPlease check reference in {1}", Product.FirstChild.OwnerDocument.BaseURI, docInstallationRequirements.BaseURI);
                        this.CheckResults.Add(ExMsg);
                        problemOccurred = true;
                        return;
                    }

                }

                if (!this.ProductAlreadyInstalled)
                {
                    XmlNodeList InstallRequirements = docProduct.SelectNodes(searchpathInstallRequirements);

                    if (InstallRequirements.Count == 0)
                    {
                        this.InstallCheckSuccessfull = true;
                        this.CheckResults.Add("\nProduct does not depend on others.");
                        return;
                    }
                    if (InstallationReferenceCheck_Success(InstallRequirements))
                    {
                        this.CheckResults.Add("\nDependency-Check:\n-----");
                        try
                        {
                            this.InstallCheckSuccessfull = CheckInstallationRequirements();
                        }
                        catch
                        {
                            this.problemOccurred = true;
                        }
                    }
                    else
                    {
                        string ExMsg = string.Format("Invalid Installation Requirement found in {0} (InstallRequirements).\r\nPlease check reference in {1}", Product.FirstChild.OwnerDocument.BaseURI, docInstallationRequirements.BaseURI);
                        this.CheckResults.Add(ExMsg);
                        problemOccurred = true;
                        return;
                    }
                }
            #endregion
            }
        }

        private bool CheckInstallationRequirements()
        {
            bool success = false;
            bool result = true;

            foreach (XmlNode Requirement in this.InstallationNodes)
            {
                string comment = string.Empty;
                string SpecialFolder = string.Empty;
                string SearchPath = string.Empty;
                string attribSearchPath = string.Empty;
                XmlNode specialFolderNode = null;
                string filename = string.Empty;

                try
                {
                    switch (Requirement.Name)
                    {

                        #region RegistryCheck
                        case "RegistryCheck":
                            object o = Registry.GetValue(Requirement.Attributes["Key"].Value, Requirement.Attributes["Value"].Value, null);
                            if (o == null)
                            {
                                success = false;
                                break;
                            }
                            switch (Requirement.FirstChild.Attributes["Method"].Value)
                            {
                                case "ValueGreaterThan":
                                    if (Convert.ToInt32(o) > Convert.ToInt32(Requirement.FirstChild.InnerText)) success = true;
                                    break;
                                case "ValueLessThan":
                                    if (Convert.ToInt32(o) < Convert.ToInt32(Requirement.FirstChild.InnerText)) success = true;
                                    break;
                                case "ValueEqualTo":
                                    if (o.ToString() == Requirement.FirstChild.InnerText) success = true;
                                    break;
                                case "ValueNotEqualTo":
                                    if (o.ToString() != Requirement.FirstChild.InnerText) success = true;
                                    break;
                                case "ValueExists":
                                    if (o.ToString() != string.Empty) success = true;
                                    break;
                                default:
                                    success = false;
                                    break;
                            }

                            break;
                        #endregion

                        #region FileCheck
                        case "FileCheck":
                            success = false;
                            SpecialFolder = string.Empty;
                            SearchPath = string.Empty;
                            filename = string.Empty;
                            attribSearchPath = string.Empty;
                            specialFolderNode = null;
                            attribSearchPath = Requirement.Attributes["SearchPath"].Value;
                            specialFolderNode = Requirement.Attributes.GetNamedItem("SpecialFolder");
                            if (specialFolderNode != null)
                            {
                                SpecialFolder = specialFolderNode.Value;
                                SearchPath = Path.GetFullPath(string.Format(@"{0}\{1}", ResolveSpecialFolder(SpecialFolder), attribSearchPath));
                            }
                            else
                            {
                                SearchPath = Path.GetFullPath(string.Format(@"{0}", attribSearchPath));
                            }
                            filename = string.Format(@"{0}\{1}", SearchPath, Requirement.Attributes["FileName"].Value);
                            if (File.Exists(filename)) success = true;
                            break;
                        #endregion

                        #region ExternalCheck
                        case "ExternalCheck":
                            success = false;
                            SpecialFolder = string.Empty;
                            SearchPath = string.Empty;
                            filename = string.Empty;
                            attribSearchPath = string.Empty;
                            specialFolderNode = null;
                            attribSearchPath = Requirement.Attributes["SearchPath"].Value;
                            specialFolderNode = Requirement.Attributes.GetNamedItem("SpecialFolder");
                            if (specialFolderNode != null)
                            {
                                SpecialFolder = specialFolderNode.Value;
                                SearchPath = Path.GetFullPath(string.Format(@"{0}\{1}", ResolveSpecialFolder(SpecialFolder), attribSearchPath));
                            }
                            else
                            {
                                SearchPath = Path.GetFullPath(string.Format(@"{0}", attribSearchPath));
                            }
                            filename = string.Format(@"{0}\{1}", SearchPath, Requirement.Attributes["CheckFile"].Value);
                            string arguments = string.Format("{0}", Requirement.Attributes["Arguments"].Value);
                            try
                            {
                                ProcessStartInfo startInfo = new ProcessStartInfo(filename);
                                startInfo.WorkingDirectory = SearchPath;
                                startInfo.Arguments = arguments;
                                startInfo.UseShellExecute = false;
                                startInfo.WindowStyle = ProcessWindowStyle.Hidden;
                                startInfo.CreateNoWindow = true;
                                startInfo.RedirectStandardError = true;
                                startInfo.RedirectStandardOutput = true;
                                Process CheckRequirementProcess = new Process();
                                CheckRequirementProcess.StartInfo = startInfo;
                                CheckRequirementProcess.Start();
                                //wait for maximal 10seconds for this check.
                                CheckRequirementProcess.WaitForExit(10000);
                                string output = CheckRequirementProcess.StandardOutput.ReadToEnd();
                                string compareMethod = Requirement.FirstChild.Attributes["Method"].Value;
                                string compareValue = Requirement.FirstChild.InnerText;
                                switch (compareMethod)
                                {
                                    case "ValueContains":
                                        if (output.Contains(compareValue)) success = true;
                                        break;
                                }
                            }
                            catch
                            {
                                success = false;
                            }
                            break;
                        #endregion

                        #region MsiProductCheck
                        case "MsiProductCheck":
                            Guid searchedGUID = new Guid(Requirement.Attributes["GUID"].Value);
                            success = this.SearchMsiProduct(searchedGUID);

                            break;
                        #endregion
                        default:
                            CheckResults.Add(string.Format("Check Time: {0}\nInvalid Installation Requirement: '{1}' found in: {2}!", this.LastCheckTime.ToLongTimeString(), Requirement.Name, Requirement.BaseURI));
                            success = false;
                            throw new InvalidInputException(string.Format("Check Time: {0}\nInvalid Installation Requirement: '{1}' found in: {2}!", this.LastCheckTime.ToLongTimeString(), Requirement.Name, Requirement.BaseURI));
                            break;
                    }
                }
                catch (SystemException ex)
                {
                    CheckResults.Add(string.Format("Check Time: {3}\nInvalid Installation Requirement '{0}' - {2} found in {1}!", Requirement.Name, Requirement.BaseURI, Requirement.Attributes["Value"].Value, this.LastCheckTime.ToLongTimeString()));
                    success = false;
                    throw new InvalidInputException(string.Format("Check Time: {3}\nInvalid Installation Requirement '{0}' - {2} found in {1}!", Requirement.Name, Requirement.BaseURI, Requirement.Attributes["Value"].Value, this.LastCheckTime.ToLongTimeString()));
                }

                switch (success)
                {
                    case true:
                        this.CheckResults.Add(string.Format("ok: {0}", Requirement.Attributes["Property"].Value));
                        result &= success;
                        break;
                    case false:
                        this.CheckResults.Add(string.Format("missing: {0}", Requirement.Attributes["Property"].Value));
                        this.MissingRequirements.Add(Requirement.Attributes["Property"].Value);
                        result &= success;
                        break;
                }
            }

            return result;
        }

        private string ResolveSpecialFolder(string FolderValue)
        {
            string resolvedValue = string.Empty;
            string[] FolderValueSplits = FolderValue.Split("%".ToCharArray());

            if (FolderValueSplits.Length == 3)
            {
                try
                {
                    switch (FolderValueSplits[1].ToString().ToUpper())
                    {
                        case "SYSTEMROOT":      //default c:\windows
                            resolvedValue = Environment.GetEnvironmentVariable("SystemRoot");
                            break;

                        case "SYSTEM32":        //default c:\windows\system32
                            resolvedValue = Environment.GetFolderPath(Environment.SpecialFolder.System);
                            break;

                        case "SYSTEMDRIVE":     //default c:
                            resolvedValue = Environment.GetEnvironmentVariable("SYSTEMDRIVE");
                            break;

                        case "WINDIR":          //default c:\windows
                            resolvedValue = Environment.GetEnvironmentVariable("WINDIR");
                            break;

                        case "ProgramFiles":    //default c:\program files
                            resolvedValue = Environment.GetEnvironmentVariable("ProgramFiles");
                            break;

                        case "TMP":    //default(vista) C:\Users\bfrank\AppData\Local\Temp
                            resolvedValue = Environment.GetEnvironmentVariable("TMP");
                            break;

                        case "TEMP":    //default(vista) C:\Users\bfrank\AppData\Local\Temp
                            resolvedValue = Environment.GetEnvironmentVariable("TEMP");
                            break;


                        default:
                            resolvedValue = Environment.GetEnvironmentVariable("SYSTEMDRIVE");
                            break;
                    }
                }
                catch (Exception)
                {
                    return string.Empty;
                }
            }

            return resolvedValue;
        }

        private bool SearchMsiProduct(Guid searchedGUID)
        {
            bool Success = false;

            // HKLM/SOFTWARE/Microsoft/Windows/CurrentVersion/Uninstall
            RegistryKey rk = Registry.LocalMachine.OpenSubKey(@"Software\Microsoft\Windows\CurrentVersion\Uninstall");
            string[] subkeynames = rk.GetSubKeyNames();
            foreach (string subkey in subkeynames)
            {
                Guid g = new Guid();
                RegistryKey relevantkey = rk.OpenSubKey(subkey);
                if (relevantkey.ValueCount > 0)
                {
                    try
                    {
                        g = new Guid(subkey.ToString());
                    }
                    catch (FormatException)
                    {
                    }
                    if (g.CompareTo(searchedGUID) == 0)
                    {
                        Success = true;
                    }
                }
            }

            return Success;
        }

        private bool InstallationReferenceCheck_Success(XmlNodeList InstallRequirements)
        {
            //loop through the installation requirements
            //every requirement for a product in hostingoptions must have a corresponding entry in installation requirements
            //if not return false
            bool referenceFound = false;
            bool result = true;

            this.InstallationNodes.Clear();

            foreach (XmlNode InstallRequirementsnode in InstallRequirements)
            {
                string RequirementName = InstallRequirementsnode.FirstChild.InnerText;
                XmlNodeList InstallRequirementProperties = docInstallationRequirements.DocumentElement.SelectNodes("//Requirements/*/@Property");
                referenceFound = false;
                foreach (XmlNode InstallRequirementPropertynode in InstallRequirementProperties)
                {
                    if (RequirementName == InstallRequirementPropertynode.InnerText)
                    {
                        referenceFound = true;
                        this.InstallationNodes.Add(InstallRequirementPropertynode.SelectSingleNode(".."));
                        break;
                    }
                }
                result &= referenceFound;
            }

            return result;
        }
    }

    #endregion

    #region UserInput
    public class UserInput
    {
        public bool Required = false;
        public List<UserInputParameter> Parameters = null;

        public UserInput(XmlNodeList Parameters)
        {
            this.Parameters = new List<UserInputParameter>();

            foreach (XmlNode input in Parameters)
            {
                //one userinput
                UserInputParameter prm = new UserInputParameter();
                this.Required = true;
                //prm.Name = input.SelectSingleNode("Input/Name").InnerText;
                //prm.InputType = input.SelectSingleNode("Input/Name").InnerText;

                string searchpath = string.Format("Input/Name");
                string searchpathType = string.Format("Input/Type");
                string searchpathDefault = string.Format("Input/Default");
                string searchpathDescription = string.Format("Input/Description");
                string searchpathRegex = string.Format("Input/RegularExpression");

                XmlNode xmlInputType = input.SelectSingleNode(searchpathType);
                XmlNode xmlInputDefault = input.SelectSingleNode(searchpathDefault);
                XmlNode xmlInputDescription = input.SelectSingleNode(searchpathDescription);
                XmlNode xmlInputRegularExpression = input.SelectSingleNode(searchpathRegex);


                if (xmlInputType == null) throw new InvalidInputException(string.Format("Invalid Input found!\r\nXmlPath='{0}'", searchpathType));
                if (xmlInputDefault == null) throw new InvalidInputException(string.Format("Invalid Input found!\r\nXmlPath='{0}')", searchpathType));
                if (xmlInputDescription == null) throw new InvalidInputException(string.Format("Invalid Input found!\r\nXmlPath='{0}')", searchpathType));


                prm.Name = input.SelectSingleNode(searchpath).InnerText;
                prm.Default = xmlInputDefault.InnerText;
                prm.Description = xmlInputDescription.InnerText;

                switch (xmlInputType.InnerText.ToLower())
                {
                    case "bool":
                        prm.InputType = UserInputType.uiTypeBool;
                        break;
                    case "string":
                        prm.InputType = UserInputType.uiTypeString;
                        if (xmlInputRegularExpression != null) prm.RegularExpression = xmlInputRegularExpression.InnerText;
                        break;
                    case "passwordstring": //renders differently (shows ****** instead of cleartext) in UserInputDialog
                        prm.InputType = UserInputType.uiTypePWDString;
                        if (xmlInputRegularExpression != null) prm.RegularExpression = xmlInputRegularExpression.InnerText;
                        break;
                    default:
                        break;
                }

                prm.ParameterXmlNode = input;
                this.Parameters.Add(prm);
            }

        }

        //private UserInputParameter GetInputType(XmlNode paramnode, UserInputParameter prm)
        //{
        //    XmlNode Action = paramnode.ParentNode.ParentNode;
        //    string searchpath = string.Format("//Input[Name='{0}']", paramnode.Attributes["src"].InnerText);
        //    string searchpathType = string.Format("{0}/Type", searchpath);
        //    string searchpathDefault = string.Format("{0}/Default", searchpath);
        //    string searchpathDescription = string.Format("{0}/Description", searchpath);
        //    string searchpathRegex = string.Format("{0}/RegularExpression", searchpath);

        //    XmlNode xmlInputType = Action.SelectSingleNode(searchpathType);
        //    XmlNode xmlInputDefault = Action.SelectSingleNode(searchpathDefault);
        //    XmlNode xmlInputDescription = Action.SelectSingleNode(searchpathDescription);
        //    XmlNode xmlInputRegularExpression = Action.SelectSingleNode(searchpathRegex);


        //    if (xmlInputType == null) throw new InvalidInputException(string.Format("Invalid Input found!\r\nXmlPath='{0}'", searchpathType));
        //    if (xmlInputDefault == null) throw new InvalidInputException(string.Format("Invalid Input found! XmlPath='{0}')", searchpathType));
        //    if (xmlInputDescription == null) throw new InvalidInputException(string.Format("Invalid Input found! XmlPath='{0}')", searchpathType));


        //    prm.Name = paramnode.Attributes["src"].InnerText;
        //    prm.Default = xmlInputDefault.InnerText;
        //    prm.Description = xmlInputDescription.InnerText;

        //    switch (xmlInputType.InnerText.ToLower())
        //    {
        //        case "bool":
        //            prm.InputType = UserInputType.logic;
        //            break;
        //        case "string":
        //            prm.InputType = UserInputType.regex;
        //            if (xmlInputRegularExpression != null) prm.RegularExpression = xmlInputRegularExpression.InnerText;
        //            break;
        //        default:
        //            break;
        //    }

        //    return prm;
        //}
    }




    public struct UserInputParameter
    {
        public string Name;
        public UserInputType InputType;
        public string Description;
        public string Default;
        public string RegularExpression;
        public object Result;
        public XmlNode ParameterXmlNode;
    }

    public enum UserInputType
    {
        uiTypeBool,
        uiTypeString,
        uiTypePWDString,
    }


    #endregion

    #region global

    [global::System.Serializable]
    public class InvalidInputException : Exception
    {

        public InvalidInputException() { }
        public InvalidInputException(string message) : base(message) { }
        public InvalidInputException(string message, Exception inner) : base(message, inner) { }
        protected InvalidInputException(
          System.Runtime.Serialization.SerializationInfo info,
          System.Runtime.Serialization.StreamingContext context)
            : base(info, context) { }
    }

    #endregion

}
